@@ -35,11 +35,7 @@
 #include "up-daemon.h"
 #include "up-device.h"
 
-#include "sysfs-utils.h"
-
 #include "up-device-supply.h"
-#include "up-device-csr.h"
-#include "up-device-unifying.h"
 #include "up-device-wup.h"
 #include "up-device-hid.h"
 #include "up-device-bluez.h"
@@ -86,6 +82,33 @@ G_DEFINE_TYPE_WITH_PRIVATE (UpBackend, up_backend, G_TYPE_OBJECT)
 static gboolean up_backend_device_add (UpBackend *backend, GUdevDevice *native);
 static void up_backend_device_remove (UpBackend *backend, GUdevDevice *native);
 
+static void
+input_switch_changed_cb (UpInput   *input,
+			 gboolean   switch_value,
+			 UpBackend *backend)
+{
+	up_daemon_set_lid_is_closed (backend->priv->daemon, switch_value);
+}
+
+static gpointer
+is_macbook (gpointer data)
+{
+	g_autofree char *product = NULL;
+
+	if (!g_file_get_contents ("/sys/devices/virtual/dmi/id/product_name", &product, NULL, NULL) ||
+	    product == NULL)
+		return GINT_TO_POINTER(FALSE);
+	return GINT_TO_POINTER(g_str_has_prefix (product, "MacBook"));
+}
+
+gboolean
+up_backend_needs_poll_after_uevent (void)
+{
+	static GOnce dmi_once = G_ONCE_INIT;
+	g_once (&dmi_once, is_macbook, NULL);
+	return GPOINTER_TO_INT(dmi_once.retval);
+}
+
 static UpDevice *
 up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 {
@@ -100,6 +123,9 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 
 		/* are we a valid power supply */
 		device = UP_DEVICE (up_device_supply_new ());
+		g_object_set (G_OBJECT(device),
+			      "ignore-system-percentage", GPOINTER_TO_INT (is_macbook (NULL)),
+			      NULL);
 		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
 		if (ret)
 			goto out;
@@ -107,16 +133,6 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 		/* no valid power supply object */
 		g_clear_object (&device);
 
-	} else if (g_strcmp0 (subsys, "hid") == 0) {
-
-		/* see if this is a Unifying mouse or keyboard */
-		device = UP_DEVICE (up_device_unifying_new ());
-		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
-		if (ret)
-			goto out;
-		/* no valid power supply object */
-		g_clear_object (&device);
-
 	} else if (g_strcmp0 (subsys, "tty") == 0) {
 
 		/* see if this is a Watts Up Pro device */
@@ -139,13 +155,6 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 		g_object_unref (device);
 #endif /* HAVE_IDEVICE */
 
-		/* see if this is a CSR mouse or keyboard */
-		device = UP_DEVICE (up_device_csr_new ());
-		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
-		if (ret)
-			goto out;
-		g_object_unref (device);
-
 		/* try to detect a HID UPS */
 		device = UP_DEVICE (up_device_hid_new ());
 		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
@@ -159,15 +168,18 @@ up_backend_device_new (UpBackend *backend, GUdevDevice *native)
 
 		/* check input device */
 		input = up_input_new ();
-		ret = up_input_coldplug (input, backend->priv->daemon, native);
+		ret = up_input_coldplug (input, native);
 		if (ret) {
 			/* we now have a lid */
 			up_daemon_set_lid_is_present (backend->priv->daemon, TRUE);
+			g_signal_connect (G_OBJECT (input), "switch-changed",
+					  G_CALLBACK (input_switch_changed_cb), backend);
+			up_daemon_set_lid_is_closed (backend->priv->daemon,
+						     up_input_get_switch_value (input));
 
-			/* not a power device */
+			/* not a power device, add it to the managed devices
+			 * and don't return a power device */
 			up_device_list_insert (backend->priv->managed_devices, G_OBJECT (native), G_OBJECT (input));
-
-			/* no valid input object */
 			device = NULL;
 		}
 		g_object_unref (input);
@@ -278,28 +290,6 @@ up_backend_uevent_signal_handler_cb (GUdevClient *client, const gchar *action,
 	}
 }
 
-static gpointer
-is_macbook (gpointer data)
-{
-	char *product;
-	gboolean ret = FALSE;
-
-	product = sysfs_get_string ("/sys/devices/virtual/dmi/id/", "product_name");
-	if (product == NULL)
-		return GINT_TO_POINTER(ret);
-	ret = g_str_has_prefix (product, "MacBook");
-	g_free (product);
-	return GINT_TO_POINTER(ret);
-}
-
-gboolean
-up_backend_needs_poll_after_uevent (void)
-{
-	static GOnce dmi_once = G_ONCE_INIT;
-	g_once (&dmi_once, is_macbook, NULL);
-	return GPOINTER_TO_INT(dmi_once.retval);
-}
-
 static gboolean
 is_battery_iface_proxy (GDBusProxy *interface_proxy)
 {
@@ -501,8 +491,8 @@ up_backend_coldplug (UpBackend *backend, UpDaemon *daemon)
 	GList *devices;
 	GList *l;
 	guint i;
-	const gchar *subsystems_wup[] = {"power_supply", "usb", "usbmisc", "tty", "input", "hid", NULL};
-	const gchar *subsystems[] = {"power_supply", "usb", "usbmisc", "input", "hid", NULL};
+	const gchar *subsystems_wup[] = {"power_supply", "usb", "usbmisc", "tty", "input", NULL};
+	const gchar *subsystems[] = {"power_supply", "usb", "usbmisc", "input", NULL};
 
 	backend->priv->daemon = g_object_ref (daemon);
 	backend->priv->device_list = up_daemon_get_device_list (daemon);
